//
// Test app by George Harth <george@modeblue.com>
//

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace ESignalStressTest
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private IESignal.HooksClass esignal;

		private string[] symbols =
		{
			"GE","AWE","PFE","TWX","MOT","EMC","C","TXN","HPQ","XOM","MU","F"
		};
		private symbolInfo[] symbolInfoArray = null;
			
		private System.Windows.Forms.TextBox usernameTextBox;
		private System.Windows.Forms.Button startBtn;
		private System.Windows.Forms.ListView listView1;
		private System.Windows.Forms.Label lastTimeLabel;
		private System.Windows.Forms.Label label1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			esignal = new IESignal.HooksClass();
			esignal.OnTimeSalesChanged +=new IESignal._IHooksEvents_OnTimeSalesChangedEventHandler(esignal_OnTimeSalesChanged);

			listView1.View = View.Details;
			listView1.Columns.Add("", 25, HorizontalAlignment.Left);
			listView1.Columns.Add("Symbol", 50, HorizontalAlignment.Left);
			listView1.Columns.Add("Type", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Price", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Size", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Exch", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Time", 60, HorizontalAlignment.Center);

			symbolInfoArray = new symbolInfo[symbols.Length];
			for (int idx = 0; idx < symbols.Length; ++idx) 
			{
				symbolInfoArray[idx] = new symbolInfo();
				symbolInfoArray[idx].symbol = symbols[idx];

				ListViewItem item = new ListViewItem((idx + 1).ToString());
				item.SubItems.Add(symbols[idx]);
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				listView1.Items.Add(item);
				symbolInfoArray[idx].item = item;	// Store it for faster access later
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			esignal.ReleaseAllTimeSales();

			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.startBtn = new System.Windows.Forms.Button();
			this.listView1 = new System.Windows.Forms.ListView();
			this.usernameTextBox = new System.Windows.Forms.TextBox();
			this.lastTimeLabel = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// startBtn
			// 
			this.startBtn.Location = new System.Drawing.Point(400, 8);
			this.startBtn.Name = "startBtn";
			this.startBtn.Size = new System.Drawing.Size(64, 24);
			this.startBtn.TabIndex = 1;
			this.startBtn.Text = "Start";
			this.startBtn.Click += new System.EventHandler(this.startBtn_Click);
			// 
			// listView1
			// 
			this.listView1.CausesValidation = false;
			this.listView1.FullRowSelect = true;
			this.listView1.GridLines = true;
			this.listView1.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listView1.Location = new System.Drawing.Point(8, 36);
			this.listView1.Name = "listView1";
			this.listView1.Size = new System.Drawing.Size(456, 326);
			this.listView1.TabIndex = 2;
			// 
			// usernameTextBox
			// 
			this.usernameTextBox.Location = new System.Drawing.Point(316, 8);
			this.usernameTextBox.Name = "usernameTextBox";
			this.usernameTextBox.Size = new System.Drawing.Size(80, 20);
			this.usernameTextBox.TabIndex = 0;
			this.usernameTextBox.Text = "";
			// 
			// lastTimeLabel
			// 
			this.lastTimeLabel.Location = new System.Drawing.Point(16, 10);
			this.lastTimeLabel.Name = "lastTimeLabel";
			this.lastTimeLabel.Size = new System.Drawing.Size(192, 18);
			this.lastTimeLabel.TabIndex = 3;
			this.lastTimeLabel.Text = "Last Time =";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(266, 10);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(48, 16);
			this.label1.TabIndex = 4;
			this.label1.Text = "usrname";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(472, 371);
			this.Controls.Add(this.usernameTextBox);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.lastTimeLabel);
			this.Controls.Add(this.listView1);
			this.Controls.Add(this.startBtn);
			this.Name = "Form1";
			this.Text = "Time and Sales Test";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void startBtn_Click(object sender, System.EventArgs e)
		{
			if (startBtn.Text == "Start")
			{
				esignal.SetApplication(usernameTextBox.Text);

				if (esignal.IsEntitled == 0)
				{
					MessageBox.Show("Not entitled");
				}
				else 
				{
					esignal.ReleaseAllHistory();
					esignal.ReleaseAllTimeSales();
					foreach (symbolInfo info in symbolInfoArray)
					{
						IESignal.TimeSalesFilter filter = new IESignal.TimeSalesFilter();
						filter.bFilterPrice = 0;
						filter.bFilterQuoteExchanges = 0;
						filter.bFilterTradeExchanges = 0;
						filter.bFilterVolume = 0;
						filter.bQuotes = 1;
						filter.bTrades = 1;
						filter.lNumDays = 1;
						filter.sSymbol = info.symbol;
						info.historyHandle = esignal.get_RequestTimeSales(ref filter);
					}
					startBtn.Text = "Stop";
					usernameTextBox.Enabled = false;
				}
			}
			else
			{
				esignal.ReleaseAllTimeSales();
				esignal.ClearSymbolCache();
				startBtn.Text = "Start";
				usernameTextBox.Enabled = false;
			}
		}

		private int historyHandleToIndex(int lHandle)
		{
			for (int idx = 0; idx < symbolInfoArray.Length; ++idx)
			{
				if (lHandle == symbolInfoArray[idx].historyHandle)
				{
					return idx;
				}
			}
			return -1;
		}

		private void esignal_OnTimeSalesChanged(int lHandle)
		{
			int index = historyHandleToIndex(lHandle);
			if ((index >= 0) && (esignal.get_GetNumTimeSalesRtBars(lHandle) > 0))
			{
				IESignal.TimeSalesData tsd = esignal.get_GetTimeSalesBar(lHandle, 0);
				ListViewItem item = symbolInfoArray[index].item;
				switch (tsd.DataType)
				{
					case IESignal.timeSalesDataFlags.tsdASK:
						item.SubItems[2].Text = "Ask";
						break;
					case IESignal.timeSalesDataFlags.tsdBID:
						item.SubItems[2].Text = "Bid";
						break;
					case IESignal.timeSalesDataFlags.tsdTRADE:
						item.SubItems[2].Text = "Trade";
						break;
					default:
						item.SubItems[2].Text = "--";
						break;
				}
				item.SubItems[3].Text = tsd.dPrice.ToString();
				item.SubItems[4].Text = tsd.lSize.ToString();
				item.SubItems[5].Text = tsd.sExchange;
				item.SubItems[6].Text = tsd.dtTime.ToShortTimeString();
				lastTimeLabel.Text = "Last Time = " + tsd.dtTime.ToShortTimeString() + 
					" [" + symbolInfoArray[index].symbol + "]";
			}
		}
	}

	public class symbolInfo
	{
		public int historyHandle = -1;
		public string symbol = "";
		public ListViewItem item = null;
	}
}
